#!/usr/bin/python2.4

#    Gscreen a GUI for linuxcnc cnc controller 
#    Chris Morley copyright 2012
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import gobject
import hal
import sys,os
from optparse import Option, OptionParser
import gladevcp.makepins
from gladevcp.gladebuilder import GladeBuilder
import pango
import traceback
import atexit

BASE = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), ".."))
libdir = os.path.join(BASE, "lib", "python")
datadir = os.path.join(BASE, "share", "linuxcnc")
sys.path.insert(0, libdir)
themedir = "/usr/share/themes"
xmlname = os.path.join(BASE,"gscreen/gscreen_test.glade")
xmlname2 = os.path.join(BASE,"gscreen/gscreen2.glade")
import gettext
LOCALEDIR = os.path.join(BASE, "share", "locale")
gettext.install("linuxcnc", localedir=LOCALEDIR, unicode=True)
gtk.glade.bindtextdomain("linuxcnc", LOCALEDIR)
gtk.glade.textdomain("linuxcnc")

def set_active(w, s):
	if not w: return
	os = w.get_active()
	if os != s: w.set_active(s)

def set_label(w, l):
	if not w: return
	ol = w.get_label()
	if ol != l: w.set_label(l)

def set_text(w, t):
	if not w: return
	ot = w.get_label()
	if ot != t: w.set_label(t)

import linuxcnc
import emc_interface
import mdi
import preferences

def excepthook(exc_type, exc_obj, exc_tb):
    try:
        w = app.widgets.window1
    except KeyboardInterrupt:
        sys.exit(0)
    except NameError:
        w = None
    lines = traceback.format_exception(exc_type, exc_obj, exc_tb)
    m = gtk.MessageDialog(w,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                ("Gscreen encountered an error.  The following "
                "information may be useful in troubleshooting:\n\n")
                + "".join(lines))
    m.show()
    m.run()
    m.destroy()
sys.excepthook = excepthook

# constants
X = 0;Y = 1;Z = 2;A = 3;B = 4;C = 5;U = 6;V = 7;W = 8
_ABS = 0;_REL = 1;_DTG = 2
_START = 0;_MAN = 1;_MDI = 2;_AUTO = 3
_MM = 1;_IMPERIAL = 0

class Trampoline(object):
    def __init__(self,methods):
        self.methods = methods

    def __call__(self, *a, **kw):
        for m in self.methods:
            m(*a, **kw)

class Widgets:
    def __init__(self, xml):
        self._xml = xml
    def __getattr__(self, attr):
        r = self._xml.get_object(attr)
        if r is None: raise AttributeError, "No widget %r" % attr
        return r
    def __getitem__(self, attr):
        r = self._xml.get_object(attr)
        if r is None: raise IndexError, "No widget %r" % attr
        return r

class Data:
    def __init__(self):
        self.use_screen2 = False
        self.theme_name = "Follow System Theme"
        self.abs_textcolor = ""
        self.rel_textcolor = ""
        self.dtg_textcolor = ""
        self.err_textcolor = ""
        self.window_geometry = ""
        self.window_max = ""
        self.axis_list = ("x","y","z","a","s")
        self.active_axis_buttons = [(None,None)]
        self.abs_color = (0, 65535, 0)
        self.rel_color = (65535, 0, 0)
        self.dtg_color = (0, 0, 65535)
        self.highlight_color = (65535,65535,65535)
        self.highlight_major = False
        self.display_order = (_REL,_DTG,_ABS)
        self.mode_order = (_START,_MAN,_MDI,_AUTO)
        self.hbutton_labels = ( ("Jog mode","Override","Mode 1","Mode 1","Mode 1"),("Mode 2","Mode 2","Mode 2","Mode 2","Mode 2") )
        self.active_gcodes = []
        self.active_mcodes = []
        self.x_abs = 0.0
        self.x_rel = 1.0
        self.x_dtg = -2.0
        self.y_abs = 0.0
        self.y_rel = 100.0
        self.y_dtg = 2.0
        self.z_abs = 0.0
        self.z_rel = 1.0
        self.z_dtg = 21.0
        self.a_abs = 0.0
        self.a_rel = 1.0
        self.a_dtg = 2.0
        self.x_is_homed = 0
        self.y_is_homed = 0
        self.z_is_homed = 0
        self.a_is_homed = 0
        self.spindle_request_rpm = 0
        self.spindle_dir = 0
        self.spindle_speed = 0
        self.active_spindle_command = "" # spindle command setting
        self.active_feed_command = "" # feed command setting
        self.system = 1
        self.estopped = True
        self.units = _IMPERIAL
        self.tool_in_spindle = 0
        self.flood = False
        self.mist = False
        self.machine_on = False
        self.or_limits = 0
        self.feed_override = 1.0
        self.spindle_override = 1.0
        self.max_velocity = 0.0
        self.edit_mode = False
        self.full_graphics = False
        self.file = ""
        self.file_lines = 0
        self.line = 0
        self.id = 0
        self.dtg = 0.0
        self.velocity = 0.0
        self.delay = 0.0
        self.preppedtool = None

    def __getitem__(self, item):
        return getattr(self, item)
    def __setitem__(self, item, value):
        return setattr(self, item, value)

def load_handlers(usermod,halcomp,builder,useropts):
    hdl_func = 'get_handlers'

    def add_handler(method, f):
        if method in handlers:
            handlers[method].append(f)
        else:
            handlers[method] = [f]

    handlers = {}
    for u in usermod:
        (directory,filename) = os.path.split(u)
        (basename,extension) = os.path.splitext(filename)
        if directory == '':
            directory = '.'
        if directory not in sys.path:
            sys.path.insert(0,directory)
            dbg('adding import dir %s' % directory)

        try:
            mod = __import__(basename)
        except ImportError,msg:
            print "module '%s' skipped - import error: %s" %(basename,msg)
	    continue
        dbg("module '%s' imported OK" % mod.__name__)
        try:
            # look for 'get_handlers' function
            h = getattr(mod,hdl_func,None)

            if h and callable(h):
                dbg("module '%s' : '%s' function found" % (mod.__name__,hdl_func))
                objlist = h(halcomp,builder,useropts)
            else:
                # the module has no get_handlers() callable.
                # in this case we permit any callable except class Objects in the module to register as handler
                dbg("module '%s': no '%s' function - registering only functions as callbacks" % (mod.__name__,hdl_func))
                objlist =  [mod]
            # extract callback candidates
            for object in objlist:
                dbg("Registering handlers in module %s object %s" % (mod.__name__, object))
                if isinstance(object, dict):
                    methods = dict.items()
                else:
                    methods = map(lambda n: (n, getattr(object, n, None)), dir(object))
                for method,f in methods:
                    if method.startswith('_'):
                        continue
                    if callable(f):
                        dbg("Register callback '%s' in %s" % (method, object))
                        add_handler(method, f)
        except Exception, e:
            print "gladevcp: trouble looking for handlers in '%s': %s" %(basename, e)
            traceback.print_exc()

    # Wrap lists in Trampoline, unwrap single functions
    for n,v in list(handlers.items()):
        if len(v) == 1:
            handlers[n] = v[0]
        else:
            handlers[n] = Trampoline(v)

    return handlers


class Gscreen(object): 

    def __init__(self):
        usage = "usage: %prog [options] myfile.ui"
        parser = OptionParser(usage=usage)
        parser.disable_interspersed_args()
        (opts, args) = parser.parse_args()

        try:
            # loading as a gtk.builder project
            self.xml = gtk.Builder()
            self.xml.add_from_file(xmlname)
        except:
            print "**** Gscreen GLADE ERROR:    With xml file: %s"% xmlname
            sys.exit(0)
        try:
            # loading as a gtk.builder project
            self.xml.add_from_file(xmlname2)
            self.screen2 = True
        except:
            print "**** Gscreen GLADE ERROR:    With xml file: %s"% xmlname
            self.screen2 = False
        self.widgets = Widgets(self.xml)
        self.data = Data()

        try:
            halcomp = hal.component("gscreen")
        except:
            print "*** Gscreen ERROR:    Asking for a HAL component using a name that already exists."
            sys.exit(0)
    
        panel = gladevcp.makepins.GladePanel( halcomp, xmlname, self.xml, None)
        halcomp.ready()
        # at this point, any glade HL widgets and their pins are set up.
        #handlers = load_handlers(None,halcomp,self.xml,None)
    
        #self.xml.connect_signals(handlers)

        self.xml.connect_signals( self )
        # access to saved prefernces
        self.prefs = preferences.preferences()

        #setup default stuff

        self.data.theme_name = self.prefs.getpref('gtk_theme', 'Follow System Theme', str)
        self.data.abs_textcolor = self.prefs.getpref('abs_textcolor', 'default', str)
        self.data.rel_textcolor = self.prefs.getpref('rel_textcolor', 'default', str)
        self.data.dtg_textcolor = self.prefs.getpref('dtg_textcolor', 'default', str)
        self.data.err_textcolor = self.prefs.getpref('err_textcolor', 'default', str)
        self.data.error_font_name = self.prefs.getpref('error_font', 'Sans Bold 10', str)
        self.data.window_geometry = self.prefs.getpref('window_geometry', 'default', str)
        self.data.window_max = self.prefs.getpref('window_force_max', 'False', bool)
        self.data.window2_geometry = self.prefs.getpref('window2_geometry', 'default', str)
        self.data.window2_max = self.prefs.getpref('window2_force_max', 'False', bool)
        self.data.use_screen2 = self.prefs.getpref('use_screen2', 'True', bool)
        self.widgets.use_screen2.set_active(self.data.use_screen2)

        w = self.widgets.statusbar1
        #w.set_font_name(self.data.error_font_name)
        #self.error_font = pango.FontDescription(self.data.error_font_name)
        #w.modify_font(self.error_font)
        if not self.data.err_textcolor == "default":
            print"change error color",self.data.err_textcolor
            w.modify_fg(gtk.STATE_NORMAL,gtk.gdk.color_parse(self.data.err_textcolor))
        self.widgets.statusbar1.push(1,"Status bar text                                                                       to here?")
        self.widgets.data_input.set_value(50.567)
        self.widgets.button_mode.set_label("Mode %d"% self.data.mode_order[0])

        # If there are themes then add them to combo box
        if os.path.exists(themedir):
            model = self.widgets.theme_choice.get_model()
            model.clear()
            model.append(("Follow System Theme",))
            temp = 0
            names = os.listdir(themedir)
            names.sort()
            for search,dirs in enumerate(names):
                model.append((dirs,))
                if dirs  == self.data.theme_name:
                    temp = search+1
            self.widgets.theme_choice.set_active(temp)
        # get the system wide theme
        settings = gtk.settings_get_default()
        self.data.system_theme = settings.get_property("gtk-theme-name")
        if not self.data.theme_name == "Follow System Theme":
            settings.set_string_property("gtk-theme-name", self.data.theme_name, "")

        # setup signals that can be blocked 
        cb = "axis_x"
        i = "_sighandler_axis_x"
        self.data[i] = int(self.widgets[cb].connect("clicked", self.on_axis_x_clicked))
        cb = "axis_y"
        i = "_sighandler_axis_y"
        self.data[i] = int(self.widgets[cb].connect("clicked", self.on_axis_y_clicked))
        cb = "axis_z"
        i = "_sighandler_axis_z"
        self.data[i] = int(self.widgets[cb].connect("clicked", self.on_axis_z_clicked))
        cb = "axis_a"
        i = "_sighandler_axis_a"
        self.data[i] = int(self.widgets[cb].connect("clicked", self.on_axis_a_clicked))

        cb = "button_v5"
        i = "_sighandler_mode_select"
        self.data[i] = int(self.widgets[cb].connect("button_press_event", self.on_mode_select_clicked))

        cb = "button_mode"
        i = "_sighandler_mode"
        self.data[i] = int(self.widgets[cb].connect("button_press_event", self.on_mode_clicked))
        cb = "gremlin"
        i = "_sighandler_gremlin"
        self.data[i] = int(self.widgets[cb].connect("button_press_event", self.on_gremlin_clicked))
        cb = "button_estop"
        i = "_sighandler_estop"
        self.data[i] = int(self.widgets[cb].connect("clicked", self.on_estop_clicked))
        for num in range(0,24):
            cb = "button_h%d"% num
            i = "_sighandler_button_h%d"% num
            self.data[i] = int(self.widgets[cb].connect("clicked", self.on_hbutton_clicked,num))
        for num in range(0,11):
            cb = "button_v%d"% num
            i = "_sighandler_button_v%d"% num
            self.data[i] = int(self.widgets[cb].connect("clicked", self.on_vbutton_clicked,num))
        self.widgets.button_v2.connect("pressed", self.on_button_v2_pressed)
        self.widgets.button_v2.connect("released", self.on_button_v2_released)
        self.widgets.button_v3.connect("pressed", self.on_button_v3_pressed)
        self.widgets.button_v3.connect("released", self.on_button_v3_released)
        self.widgets.theme_choice.connect("changed", self.on_theme_choice_changed)
        self.widgets.use_screen2.connect("clicked", self.on_use_screen2_pressed)
        self.widgets.dro_units.connect("clicked", self.on_dro_units_pressed)

        # access to EMC control
        self.emc = emc_interface.emc_control(linuxcnc, self.widgets.statusbar1)
        # access to EMC status
        self.status = emc_interface.emc_status( self.data, linuxcnc)
        # access to MDI
        mdi_labels = mdi_eventboxes = []
        self.mdi_control = mdi.mdi_control(gtk, linuxcnc, mdi_labels, mdi_eventboxes)
        # set up EMC stuff

        # check the ini file if UNITS are set to mm"
        inifile=self.emc.emc.ini(sys.argv[2])
        # first check the global settings
        units=inifile.find("TRAJ","LINEAR_UNITS")
        if units==None:
            units=inifile.find("AXIS_0","UNITS")

        if units=="mm" or units=="metric" or units == "1.0":
            self.machine_units_mm=1
            conversion=[1.0/25.4]*3+[1]*3+[1.0/25.4]*3
        else:
            self.machine_units_mm=0
            conversion=[25.4]*3+[1]*3+[25.4]*3

        self.status.set_machine_units(self.machine_units_mm,conversion)

        if self.prefs.getpref('toolsetting_fixture', 0):
            self.g10l11 = 1
        else:
            self.g10l11 = 0

        if self.prefs.getpref('dro_mm', 0):
            self.status.dro_mm(0)
        else:
            self.status.dro_inch(0)

        if self.prefs.getpref('dro_actual', 0):
           self.status.dro_actual(0)
        else:
           self.status.dro_commanded(0)

        if self.prefs.getpref('blockdel', 0):
           self.emc.blockdel_on(0)
        else:
           self.emc.blockdel_off(0)

        if self.prefs.getpref('opstop', 1):
           self.emc.opstop_on(0)
        else:
           self.emc.opstop_off(0)                   
        temp = inifile.find("TRAJ","MAX_LINEAR_VELOCITY")
        if temp == None:
            self.data._maxvelocity = 1.0
        else:
            self.data._maxvelocity = float(temp)
        print "max velocity",(float(self.data._maxvelocity) )
        # timers for updates
        gobject.timeout_add(50, self.periodic_status)
        #gobject.timeout_add(100, self.periodic_radiobuttons)

        # and display everything
        self.widgets.window1.set_title("Gscreen for linuxcnc")

        if self.screen2:
            self.widgets.window2.show()
            self.widgets.window2.move(0,0)
            if not self.data.use_screen2:
                self.widgets.window2.hide()
        self.widgets.window1.show()
        self.widgets.window1.move(300,0)
        self.update_position()
        #self.widgets.gcode_view.set_sensitive(0)
        #self.widgets.hal_led1.set_shape(2)
        self.widgets.gremlin.set_property('view','p')
        #self.widgets.gremlin.set_property('metric',True)
        self.mode_changed(self.data.mode_order[0])

# *** GLADE callbacks ****

    def on_window1_destroy(self, widget, data=None):
        print "kill"
    	gtk.main_quit()

    def on_axis_x_clicked(self,*args):
        def unclick():
            self.widgets.axis_x.handler_block(self.data._sighandler_axis_x)
            self.widgets.axis_x.set_active(False)
            self.widgets.axis_x.handler_unblock(self.data._sighandler_axis_x)

        if self.widgets.button_v1.get_active():
            self.set_axis_checks()
            unclick()
        elif self.widgets.button_v0.get_active():
            self.zero_axis()
            unclick()
        self.update_active_axis_buttons()

    def on_axis_y_clicked(self,*args):
        if self.widgets.button_v1.get_active():
         self.on_axis_set_clicked(None)
        elif self.widgets.button_v1.get_active():
         self.on_axis_zero_clicked(None)
        self.update_active_axis_buttons()

    def on_axis_z_clicked(self,*args):
        if self.widgets.button_v1.get_active():
         self.on_axis_set_clicked(None)
        if self.widgets.button_v0.get_active():
         self.on_axis_zero_clicked(None)
        self.update_active_axis_buttons()

    def on_axis_a_clicked(self,*args):
        if self.widgets.button_v1.get_active():
         self.on_axis_set_clicked(None)
        if self.widgets.button_v0.get_active():
         self.on_axis_zero_clicked(None)
        self.update_active_axis_buttons()

    def on_axis_s_clicked(self,*args):
        if self.widgets.button_v1.get_active():
         self.on_axis_set_clicked(None)
        if self.widgets.button_v0.get_active():
         self.on_axis_zero_clicked(None)
        self.update_active_axis_buttons()

    def on_mode_clicked(self,widget,event):
        # only change machine modes on click
        if event.type == gtk.gdk.BUTTON_PRESS:
            a,b,c,d = self.data.mode_order
            self.data.mode_order = b,c,d,a
            self.widgets.button_mode.set_label("Mode %d"% self.data.mode_order[0])
            modenames = ("Startup","Manual","MDI","Auto")
            self.widgets.mode_label.set_label( modenames[self.data.mode_order[0]])
            self.mode_changed(self.data.mode_order[0])

    def on_gremlin_clicked(self,widget,event):
        # only change machine modes on double click
        if event.type == gtk.gdk._2BUTTON_PRESS:
            temp = self.widgets.dro_frame.get_visible()
            temp = (temp * -1) +1
            self.widgets.dro_frame.set_visible(temp)
            self.widgets.gremlin.set_property('enable_dro',(not temp))
            print "status table ",temp

    def on_hbutton_clicked(self,widget,number):
        if number == 0: self.toggle_ignore_limits()
        elif number == 2: self.home_all()
        elif number == 4: self.unhome_all()
        elif number == 5 or number == 11: self.dro_toggle()
        elif number == 6: self.jog_mode()
        elif number == 8: self.toggle_mist()
        elif number == 9: self.toggle_flood()
        elif number == 10: self.reload_tooltable()
        elif number == 23: self.save_edit()
        else:
         print "hbutton %d clicked but no function"% int(number)

    def on_vbutton_clicked(self,widget,number):
        if number == 0:
            self.zero_axis()
        elif number == 1:
            self.set_axis_checks()
        elif number == 2:
            pass
        elif number == 3:
            pass
        elif number == 4:
            self.toggle_feed_hold()
        elif number == 9:
            self.full_graphics()
        elif number == 10:
            self.edit_mode()
        else:
         print "hbutton %d clicked but no function"% number

    def on_button_v2_pressed(self,*args):
        self.do_jog(1)
    def on_button_v2_released(self,*args):
        self.do_jog(0)
    def on_button_v3_pressed(self,*args):
        self.do_jog(-1)
    def on_button_v3_released(self,*args):
        self.do_jog(0)

    def on_mode_select_clicked(self,widget,event):
        # was it a multiple click?
        if event.type == gtk.gdk.BUTTON_PRESS:
            maxpage = self.widgets.notebook_mode.get_n_pages()
            page = self.widgets.notebook_mode.get_current_page()
            nextpage = page + 1
            print "mode select",maxpage,page,nextpage
            if nextpage == maxpage:nextpage = 0
            self.widgets.notebook_mode.set_current_page(nextpage)
        elif event.type == gtk.gdk._2BUTTON_PRESS:
            maxpage = self.widgets.notebook_main.get_n_pages()
            page = self.widgets.notebook_main.get_current_page()
            nextpage = page + 1
            print "double"
            print "mode select",maxpage,page,nextpage
            if nextpage == maxpage:nextpage = 0
            self.widgets.notebook_main.set_current_page(nextpage)

    def on_estop_clicked(self,*args):
        print "estop",self.data.estopped
        if self.data.estopped:
            self.emc.estop_reset(1)
            self.emc.machine_on(1)
        else:
            self.emc.estop(1)
            self.emc.machine_off(1)

    def on_theme_choice_changed(self,*args):
        self.change_theme()

    def on_use_screen2_pressed(self,*args):
        self.toggle_screen2()

    def on_dro_units_pressed(self,*args):
        self.toggle_dro_units()

# ****** do stuff *****

    def full_graphics(self):
        print "graphics mode"
        if self.data.full_graphics:
            self.data.full_graphics = False
            self.widgets.notebook_mode.hide()
        else:
            self.data.full_graphics = True
            self.widgets.notebook_mode.show()

    def edit_mode(self):
        print "edit mode pressed"
        if self.data.edit_mode:
            self.widgets.gcode_view.set_sensitive(0)
            self.data.edit_mode = False
            self.widgets.eventbox_gremlin.show()
            self.widgets.hbuttonbox.set_sensitive(True)
            self.widgets.button_mode.set_sensitive(True)
        else:
            self.widgets.gcode_view.set_sensitive(1)
            self.data.edit_mode = True
            self.widgets.eventbox_gremlin.hide()
            self.widgets.hbuttonbox.set_sensitive(False)
            self.widgets.button_mode.set_sensitive(False)

    def toggle_dro_units(self):
        print "toggle dro units"
        if self.data.units == _MM:
            self.status.dro_inch(1)
        else:
            self.status.dro_mm(1)

    def save_edit(self):
        print "edit"

    def update_active_axis_buttons(self):
        count = 0;temp = []
        self.data.active_axis_buttons = []
        for num,i in enumerate(self.data.axis_list):
            if self.widgets["axis_%s"%i].get_active():
                count +=1
                axisnum = num
                self.data.active_axis_buttons.append((i,num))
        if count == 0: self.data.active_axis_buttons.append((None,None))

    def jog_mode(self):
        # if muliple axis selected - unselect all of them
        if len(self.data.active_axis_buttons) > 1 and self.widgets.button_h6.get_active():
            for i in self.data.axis_list:
                self.widgets["axis_%s"%i].set_active(False)

    def do_jog(self,direction):
        # if manual mode, if jogging
        # if only one axis button pressed
        # jog positive  at selected rate
        if self.data.mode_order[0] == 1:
            if len(self.data.active_axis_buttons) > 1:
                self.widgets.statusbar1.push(2,"Can't jog multiple axis")
                print self.data.active_axis_buttons
            elif self.data.active_axis_buttons[0][0] == None:
                self.widgets.statusbar1.push(1,"No axis selected to jog")
            else:
                print "Jog axis %s" % self.data.active_axis_buttons[0][0]
                if not self.data.active_axis_buttons[0][0] == "s":
                    self.emc.jogging(1)
                    self.emc.continuous_jog_velocity(20) # TODO make selectable
                    self.emc.continuous_jog(self.data.active_axis_buttons[0][1],direction)

    def do_jog_to_position(self):
        if len(self.data.active_axis_buttons) > 1:
            self.widgets.statusbar1.push(2,"Can't jog multiple axis")
            print self.data.active_axis_buttons
        elif self.data.active_axis_buttons[0][0] == None:
            self.widgets.statusbar1.push(1,"No axis selected to jog")
        else:
            print "Jog axis %s" % self.data.active_axis_buttons[0][0]
            if not self.data.active_axis_buttons[0][0] == "s":
                self.mdi_control.go_to_position(self.data.active_axis_buttons[0][0],self.widgets.data_input.get_value(),20) #TODO config jog rate

    def toggle_screen2(self):
        self.data.use_screen2 = self.widgets.use_screen2.get_active()
        if self.screen2:
            if self.data.use_screen2:
                self.widgets.window2.show()
            else:
                self.widgets.window2.hide()
        self.prefs.putpref('use_screen2', self.data.use_screen2, bool)

    def unhome_all(self):
        self.emc.unhome_all(1)

    def home_all(self):
        self.emc.home_all(1)

    def unhome_selected(self,axis):
        pass

    def zero_axis(self):
        # cancel set mode
        if self.widgets.button_v1.get_active():
            self.widgets.button_v1.set_active(False)
        # if an axis is selected then set it
        keep = True
        for i in self.data.axis_list:
            if self.widgets["axis_%s"%i].get_active():
                keep = False
                if i == "s":
                    self.emc.spindle_off(1)
                else:
                    print "zero %s axis" %i
                    self.mdi_control.set_axis(i,0)
        if keep == False:
            self.widgets.button_v0.handler_block(self.data._sighandler_button_v0)
            self.widgets.button_v0.set_active(keep)
            self.widgets.button_v0.handler_unblock(self.data._sighandler_button_v0)

    def set_axis_checks(self):
        # cancel zero mode
        if self.widgets.button_v0.get_active():
            self.widgets.button_v0.set_active(False)
        if self.data.mode_order[0] in (_START,_MAN):
            # if in jog mode
            if self.widgets.button_h6.get_active():
                "print jog to %d"% self.widgets.data_input.get_value()
                # unpress toggle buttonn
                self.widgets.button_v1.handler_block(self.data._sighandler_button_v1)
                self.widgets.button_v1.set_active(False)
                self.widgets.button_v1.handler_unblock(self.data._sighandler_button_v1)
                self.do_jog_to_position()
            else:
                # if an axis is selected then set it
                keep = True
                for i in self.data.axis_list:
                    if self.widgets["axis_%s"%i].get_active():
                        keep = False
                        print "set %s axis" %i
                        if i == "s":
                            if self.widgets.s_display_fwd.get_active():
                                self.emc.spindle_forward(1)
                            elif self.widgets.s_display_rev.get_active():
                                self.emc.spindle_reverse(1)
                            else: continue
                            self.mdi_control.set_spindle_speed(self.widgets.data_input.get_value())
                        else:
                            self.mdi_control.set_axis(i,self.widgets.data_input.get_value())
                if keep == False:
                    self.widgets.button_v1.handler_block(self.data._sighandler_button_v1)
                    self.widgets.button_v1.set_active(keep)
                    self.widgets.button_v1.handler_unblock(self.data._sighandler_button_v1)

    def toggle_mist(self):
        if self.data.mist:
            self.emc.mist_off(1)
        else:
            self.emc.mist_on(1)

    def toggle_flood(self):
        if self.data.flood:
            self.emc.flood_off(1)
        else:
            self.emc.flood_on(1)

    def toggle_feed_hold(self):
        print "toggle feed hold",self.data.feed_hold
        if self.data.feed_hold:
            self.emc.feed_hold(0)
        else:
            self.emc.feed_hold(1)

    def toggle_ignore_limits(self):
        print "over ride limits"
        self.emc.override_limits(1)

    def reload_tooltable(self):
        self.emc.reload_tooltable(1)

    def dro_toggle(self):
        print "toggle axis display"
        a = self.data.display_order[0]
        b = self.data.display_order[1]
        c = self.data.display_order[2]
        self.data.display_order = (c,a,b)
        self.update_position()

    def mode_changed(self,mode):
        for i in range(0,4):
            if i == mode:
                self.widgets["mode%d"% i].show()
            else:
                self.widgets["mode%d"% i].hide()
        if mode == _AUTO:
            self.widgets.vmode0.hide()
            self.widgets.vmode1.show()
            self.widgets.notebook_mode.show()
            self.widgets.hal_mdihistory.hide()
        elif mode == _MDI:
            self.widgets.hal_mdihistory.show()
            self.widgets.vmode0.show()
            self.widgets.vmode1.hide()
            self.widgets.notebook_mode.hide()
        else: 
            self.widgets.vmode0.show()
            self.widgets.vmode1.hide()
            self.widgets.notebook_mode.hide()
            self.widgets.hal_mdihistory.hide()

    def change_theme(self):
        theme = self.widgets.theme_choice.get_active_text()
        self.prefs.putpref('gtk_theme', theme, str)
        if theme == "Follow System Theme":
            theme = self.data.system_theme
        settings = gtk.settings_get_default()
        settings.set_string_property("gtk-theme-name", theme, "")

    def periodic_status(self):
        self.emc.mask()
        self.emcstat = linuxcnc.stat()
        self.emcerror = linuxcnc.error_channel()
        self.emcstat.poll()
        #self.radiobutton_mask = 1
        self.status.periodic()
        self.data.system = self.status.get_current_system()
        #print self.status.data.x_abs
        #self.radiobutton_mask = 0
        e = self.emcerror.poll()
        if e:
            kind, text = e
            self.widgets.statusbar1.push(1,text)
        self.emc.unmask()
        #self.hal.periodic(self.tab == 1) # MDI tab?
        self.update_position()
        return True

    def update_position(self,*args):
        # DRO 
        for i in ("x","y","z","a","s"):
            if i == "s":
                self.widgets.s_display.set_value(abs(self.data.spindle_speed))
                self.widgets.s_display2.set_value(abs(self.data.spindle_speed))
            else:
                
                for j in range (0,3):
                    current = self.data.display_order[j]
                    attr = pango.AttrList()
                    if current == _ABS:
                        color = self.data.abs_color
                        data = self.data["%s_abs"%i]
                        #text = "%+ 10.4f"% self.data["%s_abs"%i]
                        label = "ABS"
                    elif current == _REL:
                        color = self.data.rel_color
                        data = self.data["%s_rel"%i]
                        #text = "%+ 10.4f"% self.data["%s_rel"%i]
                        label= "REL"
                    elif current == _DTG:
                        color = self.data.dtg_color
                        data = self.data["%s_dtg"%i]
                        #text = "%+ 10.4f"% self.data["%s_dtg"%i]
                        label = "DTG"
                    if j == 2:
                        if self.data.highlight_major:
                            hlcolor = self.data.highlight_color
                            bg_color = pango.AttrBackground(hlcolor[0],hlcolor[1],hlcolor[2], 0, -1)
                            attr.insert(bg_color)
                        size = pango.AttrSize(30000, 0, -1)
                        attr.insert(size)
                        weight = pango.AttrWeight(600, 0, -1)
                        attr.insert(weight)

                    fg_color = pango.AttrForeground(color[0],color[1],color[2], 0, 11)
                    attr.insert(fg_color)
                    self.widgets["%s_display_%d"%(i,j)].set_attributes(attr)
                    h = " "
                    if current == _ABS and self.data["%s_is_homed"% i]: h = "*"
                    if self.data.units == _MM:
                        text = "%s% 10.3f"% (h,data)
                    else:
                        text = "%s% 9.4f"% (h,data)
                    self.widgets["%s_display_%d"%(i,j)].set_text(text)
                    self.widgets["%s_display_%d"%(i,j)].set_alignment(0,.5)
                    self.widgets["%s_display_%d_label"%(i,j)].set_alignment(1,.5)
                    self.widgets["%s_display_%d_label"%(i,j)].set_text(label)
        # corodinate system:
        systemlabel = ("Machine","G54","G55","G56","G57","G58","G59","G59.1","G59.2","G59.3")
        # active codes
        self.widgets.active_gcodes_label.set_label(" ".join(self.data.active_gcodes))
        self.widgets.active_mcodes_label.set_label(" ".join(self.data.active_mcodes))
        self.widgets.active_feed_speed_label.set_label("F%s    S%s"% (self.data.active_feed_command,self.data.active_spindle_command))
        #tool = str(self.data.preppedtool)
        tool = str(self.data.tool_in_spindle)
        if tool == None: tool = "None"
        self.widgets.system.set_text("Tool %s     %s"%(tool,systemlabel[self.data.system]))
        # coolant
        self.widgets.led_mist.set_active(self.data.mist)
        self.widgets.led_flood.set_active(self.data.flood)
        # estop
        self.widgets.led_estop.set_active(not self.data.estopped)
        # overrides
        self.widgets.fo.set_text("FO: %d%%"%(self.data.feed_override*100))
        self.widgets.so.set_text("SO: %d%%"%(self.data.spindle_override*100))
        self.widgets.mv.set_text("MV: %d%%"%( int(self.data.max_velocity/self.data._maxvelocity *100)) )

if __name__ == "__main__":
    try:
        app = Gscreen()
        gtk.main()
    except KeyboardInterrupt:
        print "linuxcnc closed down"
        sys.exit(0)

